#!/System/Library/Frameworks/Ruby.framework/Versions/Current/usr/bin/ruby
# encoding: utf-8
require 'fileutils'
require 'rexml/document'

def class_exists?(class_name)
    klass = Module.const_get(class_name)
    return klass.is_a?(Class)
    rescue NameError
    return false
end

if class_exists? 'Encoding'
    Encoding.default_external = Encoding::UTF_8 if Encoding.respond_to?('default_external')
    Encoding.default_internal = Encoding::UTF_8 if Encoding.respond_to?('default_internal')
end

# scrivcompile.rb "path/to/doc.scriv" useheaders(true/false)

def get_children(ele,path,files,depth)
  ele.elements.each('*/BinderItem') do |child|
        # Ignore docs set to not include in compile
        includetag = REXML::XPath.first( child, "MetaData/IncludeInCompile" )
        if !includetag.nil? && includetag.text == "Yes"
          id = child.attributes["ID"]
          # passing type, would eventually use to control header/title output
          type = child.attributes["Type"]
          title = child.elements.to_a[0].text
          title = title.force_encoding('utf-8') if RUBY_VERSION.to_f > 1.9
          file = "#{path}/Files/Docs/#{id}.rtf"
          filepath = File.exists?(file) ? file : false
          files << { 'path' => filepath, 'title' => title, 'depth' => depth, 'type' => type }
        end
        get_children(child,path,files,depth+1)
  end
end

# Take the path to the scrivener file and open the internal XML file.
def get_rtf_file_array(scrivpath)
  scrivpath = File.expand_path(scrivpath)
  scrivx = File.basename("#{scrivpath}", ".scriv") + ".scrivx"
  scrivxpath = File.join(scrivpath,scrivx)
  # handle cases where the package has been renamed after creation
  unless File.exists?(scrivxpath)
    scrivxpath = %x{ls -1 #{scrivpath}/*.scrivx|head -n 1}.strip
  end
  files = []
  doc = REXML::Document.new File.new(scrivxpath)
  doc.elements.each('ScrivenerProject/Binder/BinderItem') do |ele|
    if ele.attributes['Type'] == "DraftFolder"
      get_children(ele,scrivpath,files,1)
    end
  end

  return files
end

def scrivParse(input)
  if RUBY_VERSION.to_f > 1.9
    input = input.force_encoding('utf-8')
  end
  input.gsub!(/^\s*\{\\Scrv_ps=(.*?)\\end_Scrv_ps\}\s*/m,%Q{\n<pre markdown="0"><code>\\1</code></pre>\n})
  input.gsub!(/\{\\Scrv_ps=(.*?)\\end_Scrv_ps\}/m,%Q{<code>\\1</code>})
  input.gsub!(/\{\\Scrv_annot(?:.*?)\\text=(.*?)\\end_Scrv_annot\}/m,%Q{<span class="mkscriv_annotation">\\1</span>})
  footnotes = []
  input.gsub!(/\{\\Scrv_fn=(.*?)\\end_Scrv_fn\}/m) {|match|
    link = "[^fn" + rand(99999).to_s + "]"
    footnotes.push(link + ": " + $1)
    link
  }
  return input + "\n\n" + footnotes.join("\n\n") + "\n\n"
end

path = ARGV[0].sub(/\/$/,'')
useheaders = ARGV[1] == "true" ? true : false
files = get_rtf_file_array(path)
notetext = ""
files.each{ |f|
  note = f['path'] ? %x{/usr/bin/textutil -convert txt -stdout "#{f['path']}"} : ''
  note = note.force_encoding('utf-8') if note.respond_to?('force_encoding')
  if useheaders
    leader = ""
    f['depth'].times { leader += "#" } unless f['depth'].nil?
    notetext += "#{leader} #{f['title']}\n\n" + note + "\n\n"
  else
    notetext += note + "\n\n"
  end
}
# write the result to the preview file
# watch_note = File.new("#{sw_note}",'w')
# watch_note.puts notetext
# watch_note.close
print scrivParse(notetext)
